@@ -44,10 +44,12 @@ class SendGridEmailMessageMixin:
 		
 	def _update_unique_args(self, uniqueArgs):
 		"""docstring for _update_unique_args"""
-		# assert self.unique_args is None, self.unique_args
-		self.sendgrid_headers.setUniqueArgs(uniqueArgs)
+		oldUniqueArgs = self.sendgrid_headers.data.get("unique_args", None)
+		newUniquieArgs = oldUniqueArgs.copy() if oldUniqueArgs else {}
+		newUniquieArgs.update(uniqueArgs)
+		self.sendgrid_headers.setUniqueArgs(newUniquieArgs)
 
-		return self.unique_args
+		return self.sendgrid_headers.data["unique_args"]
 
 	def update_headers(self, *args, **kwargs):
 		"""
@@ -62,12 +64,7 @@ class SendGridEmailMessageMixin:
 
 	def get_unique_args(self):
 		"""docstring for get_unique_args"""
-		if "unique_args" in self.sendgrid_headers.data:
-			# raise Exception(self.sendgrid_headers.data["unique_args"])
-			uniqueArgs = self.sendgrid_headers.data["unique_args"]
-		else:
-			uniqueArgs = None
-		return uniqueArgs
+		return self.sendgrid_headers.data.get("unique_args", None)
 	unique_args = property(get_unique_args)
 	
 	def setup_connection(self):
@@ -117,10 +114,14 @@ class SendGridEmailMessage(EmailMessage, SendGridEmailMessageMixin):
 		
 		response = super(SendGridEmailMessage, self).send(*args, **kwargs)
 		logger.debug("Tried to send an email with SendGrid and got response {r}".format(r=response))
-		sendgrid_email_sent.send(sender=self, response=response)
+		sendgrid_email_sent.send(sender=self, message=self, response=response)
 		
 		return response
 
+	def get_message_id(self):
+		return self._message_id
+	message_id = property(get_message_id)
+
 
 class SendGridEmailMultiAlternatives(EmailMultiAlternatives, SendGridEmailMessageMixin):
 	"""
@@ -138,6 +139,6 @@ class SendGridEmailMultiAlternatives(EmailMultiAlternatives, SendGridEmailMessag
 		
 		response = super(SendGridEmailMultiAlternatives, self).send(*args, **kwargs)
 		logger.debug("Tried to send an email with SendGrid and got response {r}".format(r=response))
-		sendgrid_email_sent.send(sender=self, response=response)
+		sendgrid_email_sent.send(sender=self, message=self, response=response)
 
 		return response
